@@ -7,8 +7,8 @@ test_that("CSS unit validation", {
   }
 
   # Test strings and expected results
-  strings  <- c("100x", "10px", "10.4px", ".4px", "1px0", "px", "5", "%", "5%", "auto", "1auto", "")
-  expected <- c(NA,     "10px", "10.4px", ".4px", NA,     NA,   NA,  NA,  "5%", "auto", NA,      NA)
+  strings  <- c("100x", "10px", "10.4px", ".4px", "1px0", "px", "5",  "%", "5%", "auto", "1auto", "")
+  expected <- c(NA,     "10px", "10.4px", ".4px", NA,       NA, "5px", NA, "5%", "auto", NA,      NA)
   results <- vapply(strings, validateCssUnit_wrap, character(1), USE.NAMES = FALSE)
   expect_equal(results, expected)
 
@@ -23,19 +23,8 @@ test_that("Repeated names for selectInput and radioButtons choices", {
 
   # Select input
   x <- selectInput('id','label', choices = c(a='x1', a='x2', b='x3'), selectize = FALSE)
-  choices <- x[[2]]$children
-
-  expect_equal(choices[[1]]$children[[1]], 'a')
-  expect_equal(choices[[1]]$attribs$value, 'x1')
-  expect_equal(choices[[1]]$attribs$selected, 'selected')
-
-  expect_equal(choices[[2]]$children[[1]], 'a')
-  expect_equal(choices[[2]]$attribs$value, 'x2')
-  expect_equal(choices[[2]]$attribs$selected, NULL)
-
-  expect_equal(choices[[3]]$children[[1]], 'b')
-  expect_equal(choices[[3]]$attribs$value, 'x3')
-  expect_equal(choices[[3]]$attribs$selected, NULL)
+  expect_equal(format(x), '<label class="control-label" for="id">label</label>
+<select id="id"><option value="x1" selected>a</option>\n<option value="x2">a</option>\n<option value="x3">b</option></select>')
 
 
   # Radio buttons
